// SPDX-License-Identifier: GPL-3.0
pragma solidity ^0.8.23;

import "@openzeppelin/contracts/access/AccessControl.sol";

contract DappIndexer is AccessControl {
    // Moderator adds/updates and removes DAPPs
    bytes32 public constant MODERATOR_ROLE = keccak256("MODERATOR_ROLE");

    struct PackedCID {
        bytes32 CID1;
        bytes32 CID2;
    }

    mapping (bytes32 => PackedCID) private dapps;

    bytes32[] public dapp_names;

    constructor(
        address defaultAdmin,
        address moderator
    ){
        _grantRole(DEFAULT_ADMIN_ROLE, defaultAdmin);
        _grantRole(MODERATOR_ROLE, moderator);
    }

    function addDapp(bytes32 dapp_name, PackedCID calldata cid)
    external onlyRole(MODERATOR_ROLE) {
        //if the struct does not exists trace it on the array
        if (dapps[dapp_name].CID1 != 0 && dapps[dapp_name].CID2 != 0) {
            dapp_names.push(dapp_name);
        }

        dapps[dapp_name] = PackedCID(cid.CID1, cid.CID2); 
    }

    //@Notice: consider calling getDappIndex internal and pop dapps and dapp_names
    //@Notice: Yet requiring index might give extra verification
    function removeDapp(uint index, bytes32 dapp_name) 
    external onlyRole(MODERATOR_ROLE) {
        require(index < dapp_names.length, "dapp index out of bounds");
        require(dapp_names[index] == dapp_name);
        dapp_names[index] = dapp_names[dapp_names.length - 1];
        dapp_names.pop();
        delete dapps[dapp_name];
    }

    function getDapp(bytes32 dapp_name) external view returns (PackedCID memory){
        return dapps[dapp_name]; 
    }

    function getDappIndex(bytes32 dapp_name) external view returns (uint256 i) {
       require(dappExists(dapp_name));
       for(i; i <= dapp_names.length; i++){
           if(dapp_names[i] == dapp_name){
               return i;
           }
       }
    }

    function listDappNames() external view returns (bytes32[] memory) {
        return dapp_names;
    }

    function dappExists(bytes32 dapp_name) public view returns (bool) {
       for(uint i; i <= dapp_names.length; i++){
           if(dapp_names[i] == dapp_name){
               return true;
           }
       }
       return false;
    }

}
